# The LearningOnline Network
# RSS Feeder
#
# $Id: lonrss.pm,v 1.27 2006/08/11 20:11:47 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonrss;

use strict;
use LONCAPA;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonlocal;
use Apache::lonhtmlcommon;


sub filterfeedname {
    my $filename=shift;
    $filename=~s/(\_rss\.html|\.rss)$//;
    $filename=~s/\W//g;
    $filename=~s/\_rssfeed$//;
    $filename=~s/^nohist\_//;
    return $filename;
}

sub feedname {
    return 'nohist_'.&filterfeedname(shift).'_rssfeed';
}

sub displayfeedname {
    my ($rawname,$uname,$udom)=@_;
    my $filterfilename=&filterfeedname($rawname);
# do we have a stored name?
    my %stored=&Apache::lonnet::get('nohist_all_rss_feeds',[$filterfilename,'feed_display_option_'.$filterfilename],$udom,$uname);
    if ($stored{$filterfilename}) { return ($stored{$filterfilename},$stored{'feed_display_option_'.$filterfilename}); }
# no, construct a name
    my $name=$filterfilename; 
    if ($name=~/^CourseBlog/) {
        $name=&mt('Course Blog');
	if ($env{'course.'.$env{'request.course.id'}.'.description'}) {
	    $name.=' '.$env{'course.'.$env{'request.course.id'}.'.description'};
	}
    } else {
	$name=~s/\_/ /g;
    }
    return ($name,$stored{'feed_display_option_'.$filterfilename});
}

sub namefeed {
    my ($rawname,$uname,$udom,$newname)=@_;
    return &Apache::lonnet::put('nohist_all_rss_feeds',
				{ &filterfeedname($rawname) => $newname },
				$udom,$uname);
}

sub changefeeddisplay {
    my ($rawname,$uname,$udom,$newstatus)=@_;
    return &Apache::lonnet::put('nohist_all_rss_feeds',
				{ 'feed_display_option_'.&filterfeedname($rawname) => $newstatus },
				$udom,$uname);
}

sub advertisefeeds {
    my ($uname,$udom,$edit)=@_;
    my $feeds='';
    my %feednames=&Apache::lonnet::dump('nohist_all_rss_feeds',$udom,$uname);
    my $mode='public';
    if ($edit) {
	$mode='adm';
    }
    foreach my $feed (sort(keys(%feednames))) {
	if (($feed!~/^error\:/) && ($feed!~/^feed\_display\_option\_/)) {
	    my $feedurl='http://'.$ENV{'HTTP_HOST'}.'/public/'.$udom.'/'.$uname.'/'.$feed.'.rss';
	    my $htmlurl='http://'.$ENV{'HTTP_HOST'}.'/'.$mode.'/'.$udom.'/'.$uname.'/'.$feed.'_rss.html';
	    if ($feednames{'feed_display_option_'.$feed} eq 'hidden') {
		if ($edit) {
		    $feeds.='<li><i>'.$feednames{$feed}.'</i><br />'.&mt('Hidden').': <a href="'.$htmlurl.'"><tt>'.$htmlurl.'</tt></a></li>';
		}
	    } else {
		$feeds.='<li><b>'.$feednames{$feed}.
		    '</b><br />'.($edit?&mt('Edit'):'HTML').': <a href="'.$htmlurl.'"><tt>'.$htmlurl.'</tt></a>'.
		    '<br />RSS: <a href="'.$feedurl.'"><tt>'.$feedurl.'</tt></a></li>';
	    }
	}
    }
    if ($feeds) {
	return '<h4>'.&mt('Available RSS Feeds and Blogs').'</h4><ul>'.$feeds.'</ul>';
    } else {
        return '';
    }
}

sub rss_link {
    my ($url) = @_;
    return qq|<link rel="alternate" type="application/rss+xml" title="Course Announcements" href="$url" />|;
}

{
    my $feedcounter;
    sub get_new_feed_id {
	$feedcounter++;
	return time().'00000'.$$.'00000'.$feedcounter;
    }
}

sub addentry {
    my $id=&get_new_feed_id();
    return &editentry($id,@_);
}

sub editentry {
    my ($id,$uname,$udom,$filename,$title,$description,$url,$status,$encurl,$enctype)=@_;
    if ($status eq 'deleted') {
	return &changestatus($id,$uname,$udom,$filename,$status);
    }
    my $feedname=&feedname($filename);
    &Apache::lonnet::put('nohist_all_rss_feeds',
			 { &filterfeedname($filename) => 
			       (&displayfeedname($filename,$uname,$udom))[0] },
			 $udom,$uname);
    return &Apache::lonnet::put($feedname,{
	$id.'_title' => $title,
	$id.'_description' => $description,
	$id.'_link' => $url,
	$id.'_enclosureurl' => $encurl,
	$id.'_enclosuretype' => $enctype,
	$id.'_status' => $status},$udom,$uname);
}

sub changestatus {
    my ($id,$uname,$udom,$filename,$status)=@_;
    my $feedname=&feedname($filename);
    if ($status eq 'deleted') {
	return &Apache::lonnet::del($feedname,[$id.'_title',
					       $id.'_description',
					       $id.'_link',
					       $id.'_enclosureurl',
					       $id.'_enclosuretype',
					       $id.'_status'],$udom,$uname);
    } else {
	return &Apache::lonnet::put($feedname,{$id.'_status' => $status},$udom,$uname);
    }
}

sub changed_js {
    return <<ENDSCRIPT;
<script type="text/javascript">
    function changed(tform,id) {
        tform.elements[id+"_modified"].checked=true;
    }
</script>
ENDSCRIPT
}

sub determine_enclosure_types {
    my ($url)=@_;
    my ($ending)=($url=~/\.(\w+)$/);
    return &Apache::loncommon::filemimetype($ending);
}

sub course_blog_link {
    my ($id,$title,$description,$url,$encurl,$enctype)=@_;
    if ($env{'request.course.id'}) {
	return &add_blog_entry_link($id,
				    $env{'course.'.$env{'request.course.id'}.'.num'},
				    $env{'course.'.$env{'request.course.id'}.'.domain'},
				    'Course_Announcements',
				    $title,$description,$url,'public',$encurl,$enctype,
				    &mt('Add to Course Announcements'));
    } else {
	return '';
    }
}

sub add_blog_entry_link {
    my ($id,$uname,$udom,$filename,$title,$description,$url,$status,$encurl,$enctype,$linktext)=@_;
    return "<a href='/adm/$udom/$uname/".&filterfeedname($filename).'_rss.html?queryid='.
	&escape($id).
	'&amp;title='.&escape($title).
	'&amp;description='.&escape($description).
	'&amp;url='.&escape($url).
	'&amp;status='.&escape($status).
	'&amp;encurl='.&escape($encurl).
	'&amp;enctype='.&escape($enctype).
	"'>".$linktext.'</a>';

}

sub handler {
    my ($r) = @_;

    my $edit=0;
    my $html=0;
    my (undef,$mode,$udom,$uname,$filename)=split(/\//,$r->uri);
    if (($mode eq 'adm') && ($env{'user.name'} eq $uname) && ($env{'user.domain'} eq $udom)) {
	$edit=1;
	$html=1;
    }
    if  (($mode eq 'adm') && (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
	$edit=1;
	$html=1;
    }
    if ($filename=~/\.html$/) {
	$html=1;
    }
    if ($html) {
	&Apache::loncommon::content_type($r,'text/html');
    } else {
# Workaround Mozilla/Firefox
#	&Apache::loncommon::content_type($r,'application/rss+xml');
	&Apache::loncommon::content_type($r,'text/xml');
    }
    $r->send_http_header;
    return OK if $r->header_only;

    my $filterfeedname=&filterfeedname($filename);
    my $feedname=&feedname($filename);
    my ($displayfeedname,$displayoption)=&displayfeedname($filename,$uname,$udom);
    if ($html) {
	my $title = $displayfeedname?$displayfeedname
                                    :"Available RSS Feeds and Blogs";
	$r->print(&Apache::loncommon::start_page($title,undef,
						 {'domain'         => $udom,
						  'force_register' =>
						      $env{'form.register'}}).
		  &changed_js());
    } else { # render RSS
	$r->print("<rss version='2.0' xmlns:dc='http://purl.org/dc/elements/1.1'>\n<channel>".
		  "\n<link>http://".$ENV{'HTTP_HOST'}.'/public/'.$udom.'/'.$uname.'/'.
		  $filterfeedname.'_rss.html</link>'.
		  "\n<description>".
		  &mt('An RSS Feed provided by the LON-CAPA Learning Content Management System').
		  '</description>');
    }
# This will be the entry id for new additions to the blog
    my $newid = &get_new_feed_id();
# Is this user for real?
    my $homeserver=&Apache::lonnet::homeserver($uname,$udom);
    if ($html) {
# Any new feeds or renaming of feeds?
	if ($edit) {
# Hide a feed?
	    if ($env{'form.hidethisblog'}) {
		&changefeeddisplay($feedname,$uname,$udom,'hidden');
		($displayfeedname,$displayoption)=&displayfeedname($filename,$uname,$udom);
	    }
# Advertise a feed?
	    if ($env{'form.advertisethisblog'}) {
		&changefeeddisplay($feedname,$uname,$udom,'public');
		($displayfeedname,$displayoption)=&displayfeedname($filename,$uname,$udom);
	    }
# New feed?
	    if ($env{'form.namenewblog'}=~/\w/) {
		&namefeed($env{'form.namenewblog'},$uname,$udom,$env{'form.namenewblog'});
	    }
# Old feed that is being renamed?
	    if (($displayfeedname) && ($env{'form.newblogname'}=~/\w/)) {
		if ($env{'form.newblogname'} ne $displayfeedname) {
		    &namefeed($feedname,$uname,$udom,$env{'form.newblogname'});
		    ($displayfeedname,$displayoption)=&displayfeedname($filename,$uname,$udom);
		}
	    }
	}
	$r->print(&advertisefeeds($uname,$udom,$edit));
    } 
    if ($homeserver eq 'no_host') {
	$r->print(($html?'<h3>':'<title>').&mt('No feed available').($html?'</h3>':'</title>'));
    } else { # is indeed a user
# Course or user?
	my $name='';
	if ($uname=~/^\d/) {
	    my %cenv=&Apache::lonnet::dump('environment',$udom,$uname);
	    $name=$cenv{'description'};
	} else {
	    $name=&Apache::loncommon::nickname($uname,$udom);
	}
# Add a new feed
        if (($html) && ($edit)) {
	    $r->print('<form method="post">');
            $r->print(&mt('Name for New Feed').": <input type='text' size='40' name='namenewblog' />");
	    $r->print('<input type="submit" value="'.&mt('Start a New Feed').'" />');
	    $r->print('</form>');
	}
        if ($displayfeedname) { # this is an existing feed
# Anything to store?
	    if ($edit) {
# check if this was called with a query string
		&Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['queryid']);
		if ($env{'form.queryid'}) {
# yes, collect the remainder
		    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
							    ['title',
							     'description',
							     'url',
							     'status',
							     'enclosureurl',
							     'enclosuretype']);
#    my ($id,$uname,$udom,$filename,$title,$description,$url,$status,$encurl,$enctype)=@_;

		    &editentry($env{'form.queryid'},
			       $uname,$udom,$filename,
			       $env{'form.title'},
			       $env{'form.description'},
			       $env{'form.url'},
			       $env{'form.status'},
			       $env{'form.encurl'},
			       $env{'form.enctype'}
			       );
		}
		my %newsfeed=&Apache::lonnet::dump($feedname,$udom,$uname);
		foreach my $entry (sort(keys(%newsfeed)),$env{'form.newid'}.'_status') {
		    if ($entry=~/^(\d+)\_status$/) {
			my $id=$1;
			if ($env{'form.'.$id.'_modified'}) {
			    &editentry($id,$uname,$udom,$feedname,
				       $env{'form.'.$id.'_title'},
				       $env{'form.'.$id.'_description'},
				       $env{'form.'.$id.'_url'},
				       $env{'form.'.$id.'_status'},
				       $env{'form.'.$id.'_enclosureurl'},
				       $env{'form.'.$id.'_enclosuretype'},
				       );
			}
		    }
		}
	    } #done storing

	    $r->print("\n".
		      ($html?'<hr /><h3>':'<title>').
		      &mt('LON-CAPA Feed "[_1]" for [_2]',$displayfeedname,$name).
		      ($displayoption eq 'hidden'?' ('.&mt('Hidden').')':'').
		      ($html?'</h3>'.($edit?'<form method="post"><br />'.
				      &mt('Name of this Feed').
				      ': <input type="text" size="50" name="newblogname" value="'.
				      $displayfeedname.'" />':'').'<ul>':'</title>'));
# Render private items?
	    my $viewpubliconly=1;
	    if (($env{'user.name'} eq $uname) && ($env{'user.domain'} eq $udom)) {
		$viewpubliconly=0;
	    }
# Get feed items
	    my %newsfeed=&Apache::lonnet::dump($feedname,$udom,$uname);
	    foreach my $entry (sort(keys(%newsfeed)),$newid.'_status') {
		if ($entry=~/^(\d+)\_status$/) { # is an entry
		    my $id=$1;
		    if ($edit) {
			my %lt=&Apache::lonlocal::texthash('public' => 'public',
							   'private' => 'private',
							   'hidden' => 'hidden',
							   'delete' => 'delete',
							   'store' => 'Store changes',
							   'title' => 'Title',
							   'link' => 'Link',
							   'description' => 'Description');
			my %status=();
			unless ($newsfeed{$id.'_status'}) { $newsfeed{$id.'_status'}='public'; }
			$status{$newsfeed{$id.'_status'}}='checked="checked"';
			$r->print(<<ENDEDIT);
<li>
<label><input name='$id\_modified' type='checkbox' value="modified" /> $lt{'store'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="public" $status{'public'} onClick="changed(this.form,'$id');" /> $lt{'public'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="private" $status{'private'} onClick="changed(this.form,'$id');" /> $lt{'private'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="hidden" $status{'hidden'} onClick="changed(this.form,'$id');" /> $lt{'hidden'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="deleted" onClick="changed(this.form,'$id');" /> $lt{'delete'}</label>
<br />
$lt{'title'}:
<input name='$id\_title' type='text' size='60' value='$newsfeed{$id.'_title'}' onChange="changed(this.form,'$id');" /><br />
$lt{'description'}:<br />
<textarea name='$id\_description' rows="6" cols="80" onChange="changed(this.form,'$id');">$newsfeed{$id.'_description'}</textarea><br />
$lt{'link'}:
<input name='$id\_link' type='text' size='60' value='$newsfeed{$id.'_link'}' onChange="changed(this.form,'$id');" />
<hr /></li>
ENDEDIT
		    } else { # not in edit mode, just displaying
			if (($newsfeed{$id.'_status'} ne 'public') && ($viewpubliconly)) { next; }
			if ($newsfeed{$id.'_status'} eq 'hidden') { next; }
			$r->print("\n".($html?"\n<li><b>":"<item>\n<title>").$newsfeed{$id.'_title'}.
				  ($html?"</b><br />\n":"</title>\n<description>").
				  $newsfeed{$id.'_description'}.
				  ($html?"<br />\n<a href='":"</description>\n<link>").
				  "http://".$ENV{'HTTP_HOST'}.
				  $newsfeed{$id.'_link'}.
				  ($html?("'>".&mt('Read more')."</a><br />\n"):"</link>\n"));
# Enclosure? Get stats
			if ($newsfeed{$id.'_enclosureurl'}) {
			    my @stat=&Apache::lonnet::stat_file($newsfeed{$id.'_enclosureurl'});
			    if ($stat[7]) {
# Has non-zero length (and exists)
				my $enclosuretype=$newsfeed{$id.'_enclosetype'};
				$r->print(($html?"<a href='":"\n<enclosure url='").
					  $newsfeed{$id.'_enclosureurl'}."' length='".$stat[7].
					  "' type='".$enclosuretype.($html?"'>".&mt('Enclosure')."</a>":"' />"));
			    }
			}
			if ($html) { # is HTML
			    $r->print("\n<hr /></li>\n");
			} else { # is RSS
			    $r->print("\n<guid isPermaLink='false'>".$id.$filterfeedname.'_'.$udom.'_'.$uname."</guid></item>\n");
			}
		    } # end of "in edit mode"
		} # end of rendering a real entry
	    } # end of loop through all keys
	    if ($html) {
		$r->print('</ul>');
		if ($edit) {
		    $r->print('<input type="hidden" name="newid" value="'.$newid.'"/><input type="submit" value="'.&mt('Store Marked Changes').'" />'.
			      ($displayoption eq 'hidden'?'<input type="submit" name="advertisethisblog" value="'.&mt('Advertise this Feed').'" />':
			       '<input type="submit" name="hidethisblog" value="'.&mt('Hide this Feed').'" />'));
		}
	    }
	} # was a real display feedname
	$r->print(($html?'</form>'.&Apache::loncommon::end_page():'</channel></rss>'."\n"));
    } # a real user
    return OK;
} # end handler
1;
__END__
